@@ -81,6 +81,9 @@ module Hledger.Cli.CliOptions (
 
   -- * Other utils
   topicForMode,
+  DeclarablesSelector(..),
+  declarablesSelectorFromOpts,
+  findMatchedByArgument,
 
 --  -- * Convenience re-exports
 --  module Data.String.Here,
@@ -88,17 +91,17 @@ module Hledger.Cli.CliOptions (
 )
 where
 
-import qualified Control.Exception as C
+import Control.Exception qualified as C
 import Control.Monad (when)
 import Data.Char
 import Data.Default
 import Data.List.Extra (intercalate, isInfixOf, nubSort)
-import qualified Data.List.NonEmpty as NE (NonEmpty, fromList, nonEmpty)
+import Data.List.NonEmpty qualified as NE (NonEmpty, fromList, nonEmpty)
 import Data.List.Split (splitOn)
 import Data.Maybe
 --import Data.String.Here
 -- import Data.Text (Text)
-import qualified Data.Text as T
+import Data.Text qualified as T
 import Data.Void (Void)
 import GitHash (tGitInfoCwdTry)
 import Safe
@@ -118,7 +121,8 @@ import Hledger
 import Hledger.Cli.DocFiles
 import Hledger.Cli.Version
 import Data.Time.Clock.POSIX (POSIXTime)
-import Data.List (isPrefixOf, isSuffixOf)
+import Data.List (find, isPrefixOf, isSuffixOf)
+import Data.Function ((&))
 
 
 -- | The name of this program's executable.
@@ -152,7 +156,14 @@ Right binaryinfo = parseHledgerVersion prognameandversion
 -- | Common input-related flags: --file, --rules, --conf, --alias...
 inputflags :: [Flag RawOpts]
 inputflags = [
-   flagReq  ["file","f"]      (\s opts -> Right $ setopt "file" s opts) "[FMT:]FILE" "Read data from FILE, or from stdin if FILE is -, inferring format from extension or a FMT: prefix. Can be specified more than once. If not specified, reads from $LEDGER_FILE or $HOME/.hledger.journal."
+   flagReq  ["file","f"]      (\s opts -> Right $ setopt "file" s opts) "[FMT:]FILE" $ unwords [
+     "Use this as the journal file (- means stdin)."
+    ,"If not specified, $LEDGER_FILE or ~/.hledger.journal will be used."
+    ,"If specified more than once, the files will be read in order."
+    ,"Each file's format (journal, csv, timeclock, timedot, rules..) is inferred from the file extension or a FMT: prefix."
+    ,"Some commands (add, import) write to the (first) file,"
+    ,"and expect it to be in journal format."
+    ]
   ,flagReq  ["rules"]         (\s opts -> Right $ setopt "rules" s opts) "RULESFILE" "Use rules defined in this rules file for converting subsequent CSV/SSV/TSV files. If not specified, uses FILE.csv.rules for each FILE.csv."  -- see also hiddenflags
 
   ,flagReq  ["alias"]         (\s opts -> Right $ setopt "alias" s opts)  "A=B|/RGX/=RPL" "transform account names from A to B, or by replacing regular expression matches"
@@ -164,6 +175,11 @@ inputflags = [
     , "In hledger-ui, also make future-dated transactions visible at startup."
     ])
   ,flagNone ["ignore-assertions","I"] (setboolopt "ignore-assertions") "don't check balance assertions by default"
+  ,flagReq  ["txn-balancing"] (\s opts -> Right $ setopt "txn-balancing" s opts) "..." (unlines [
+     "how to check that transactions are balanced:"
+    ,"'old':   - use global display precision"
+    ,"'exact': - use transaction precision (default)"
+    ])
   ,flagNone ["infer-costs"] (setboolopt "infer-costs") "infer conversion equity postings from costs"
   ,flagNone ["infer-equity"] (setboolopt "infer-equity") "infer costs from conversion equity postings"
   -- history of this flag so far, lest we be confused:
@@ -291,7 +307,8 @@ hiddenflagsformainmode = [
   ,flagNone ["pretty-tables"]        (setopt "pretty" "always") "legacy flag that was renamed"
   ,flagNone ["anon"]                 (setboolopt "anon") "deprecated, renamed to --obfuscate"  -- #2133, handled by anonymiseByOpts
   ,flagNone ["obfuscate"]            (setboolopt "obfuscate") "slightly obfuscate hledger's output. Warning, does not give privacy. Formerly --anon."  -- #2133, handled by maybeObfuscate
-  ,flagNone ["timeclock-old"]        (setboolopt "oldtimeclock") "don't pair timeclock entries by account name"
+  ,flagNone ["old-timeclock", "timeclock-old"] (setboolopt "oldtimeclock") "don't pair timeclock entries by account name"
+  ,flagNone ["old-glob"]             (setboolopt "oldglob") "deprecated, no longer used as of 1.50.4"  -- #2498
   ,flagReq  ["rules-file"]           (\s opts -> Right $ setopt "rules" s opts) "RULESFILE" "was renamed to --rules"
   ]
 
@@ -342,8 +359,8 @@ mkgeneralflagsgroups3 helpflags' = [
    (helpflagstitle, helpflags')
   ]
 
-inputflagstitle  = "\nGeneral input/data transformation flags"
-outputflagstitle = "\nGeneral output/reporting flags (supported by some commands)"
+inputflagstitle  = "\nGeneral input flags"
+outputflagstitle = "\nGeneral output flags (affecting some commands)"
 helpflagstitle   = "\nGeneral help flags"
 
 -- Several subsets of the standard general flags plus CLI help flags, as separate groups.
@@ -810,6 +827,50 @@ registerWidthsFromOpts CliOpts{width_=Just s}  =
           eof
           return (totalwidth, descwidth)
 
+-- Some common ways to select items from a list of declarable things.
+-- Used by the accounts, commodities, payees, tags commands, eg.
+data DeclarablesSelector
+  = Used
+  | Declared
+  | Undeclared
+  | Unused
+  | Find
+  deriving (Show, Eq)
+
+-- Get the flag of this kind from opts, or raise an error if there's more than one.
+declarablesSelectorFromOpts :: CliOpts -> Maybe DeclarablesSelector
+declarablesSelectorFromOpts CliOpts{rawopts_=rawopts} =
+  case ( boolopt "used"       rawopts
+       , boolopt "declared"   rawopts
+       , boolopt "undeclared" rawopts
+       , boolopt "unused"     rawopts
+       , boolopt "find"       rawopts
+       ) of
+    (False, False, False, False, False) -> Nothing
+    (True,  False, False, False, False) -> Just Used
+    (False, True,  False, False, False) -> Just Declared
+    (False, False, True,  False, False) -> Just Undeclared
+    (False, False, False, True,  False) -> Just Unused
+    (False, False, False, False, True ) -> Just Find
+    _ -> error' "please pick at most one of --used, --declared, --undeclared, --unused, --find"
+
+-- | A helper for the --find mode offered by commands like accounts, commodities, payees, tags (see also 'DeclarablesSelector').
+-- Interpret the first command argument found in rawopts as a case insensitive regular expression,
+-- then return the first of the provided items that it matches;
+-- or raise an error if there's no valid argument or no matched item.
+-- This function's second argument describes the items' type, for the error message.
+findMatchedByArgument :: RawOpts -> String -> [T.Text] -> T.Text
+findMatchedByArgument rawopts itemtype items =
+  let
+    arg = headDef err $ listofstringopt "args" rawopts
+      where err = error' $ "With --find, please provide a " ++ itemtype ++ " name or\n" ++
+              itemtype ++ " pattern (case-insensitive, infix, regexp) as first command argument."
+    firstmatch = case toRegexCI $ T.pack arg of  -- keep synced with aregister's matching
+      Right re -> find (regexMatchText re)
+      Left  _  -> const Nothing
+  in firstmatch items
+    & fromMaybe (error' $ show arg ++ " did not match any " ++ itemtype ++ ".")
+
 -- Other utils
 
 -- None of https://hackage.haskell.org/package/directory-1.3.8.1/docs/System-Directory.html#g:5
