@@ -24,10 +24,10 @@ import Data.List (intersperse, intercalate)
 import Data.List.Extra (nubSort)
 import Data.Text (Text)
 import Data.Map (Map)
-import qualified Data.Map as Map
-import qualified Data.Text as T
-import qualified Data.Text.Lazy as TL
-import qualified Data.Text.Lazy.Builder as TB
+import Data.Map qualified as Map
+import Data.Text qualified as T
+import Data.Text.Lazy qualified as TL
+import Data.Text.Lazy.Builder qualified as TB
 import Lens.Micro ((^.), _Just, has)
 import Safe (lastMay, minimumDef)
 import System.Console.CmdArgs.Explicit
@@ -37,35 +37,36 @@ import Hledger.Write.Beancount (accountNameToBeancount, showTransactionBeancount
 import Hledger.Write.Csv (CSV, printCSV, printTSV)
 import Hledger.Write.Ods (printFods)
 import Hledger.Write.Html.Lucid (styledTableHtml)
-import qualified Hledger.Write.Spreadsheet as Spr
+import Hledger.Write.Spreadsheet qualified as Spr
 import Hledger.Cli.CliOptions
 import Hledger.Cli.Utils
 import Hledger.Cli.Anchor (setAccountAnchor)
-import qualified Lucid
-import qualified System.IO as IO
+import Lucid qualified
+import System.IO qualified as IO
 import Data.Maybe (isJust, catMaybes, fromMaybe)
 import Hledger.Write.Beancount (commodityToBeancount, tagsToBeancountMetadata)
 
 printmode = hledgerCommandMode
   $(embedFileRelative "Hledger/Cli/Commands/Print.txt")
-  ([flagNone ["explicit","x"] (setboolopt "explicit")
-    "show all amounts explicitly"
-  ,flagNone ["show-costs"] (setboolopt "show-costs")
-    "show transaction prices even with conversion postings"
-  ,roundFlag
+  ([
+   flagNone ["explicit","x"] (setboolopt "explicit") "show all amounts explicitly"
   ,flagNone ["invert"] (setboolopt "invert") "display all amounts with reversed sign"
-  ,flagNone ["new"] (setboolopt "new")
-    "show only newer-dated transactions added in each file since last run"
+  ,flagNone ["locations"] (setboolopt "locations") "add tags showing file paths and line numbers"
   ,let arg = "DESC" in
    flagReq  ["match","m"] (\s opts -> Right $ setopt "match" s opts) arg
     ("fuzzy search for one recent transaction with description closest to "++arg)
-  ,flagReq  ["base-url"] (\s opts -> Right $ setopt "base-url" s opts) "URLPREFIX" "in html output, generate links to hledger-web, with this prefix. (Usually the base url shown by hledger-web; can also be relative.)"
-  ,flagNone ["location"] (setboolopt "location") "add file/line number tags to print output"
+  ,flagNone ["new"] (setboolopt "new") "show only newer-dated transactions added in each file since last run"
+  ,roundFlag
+  ,flagReq  ["base-url"] (\s opts -> Right $ setopt "base-url" s opts) "URLPREFIX"
+    "in html output, generate links to hledger-web, with this prefix. (Usually the base url shown by hledger-web; can also be relative.)"
   ,outputFormatFlag ["txt","beancount","csv","tsv","html","fods","json","sql"]
   ,outputFileFlag
   ])
   cligeneralflagsgroups1
-  hiddenflags
+  (hiddenflags ++
+  -- this works already as a unique abbreviation, but declare here in case it ever becomes nonunique
+  [flagNone ["location"] (setboolopt "locations") "deprecated, use --locations instead"
+  ])
   ([], Just $ argsFlag "[QUERY]")
 
 roundFlag = flagReq  ["round"] (\s opts -> Right $ setopt "round" s opts) "TYPE" $
@@ -113,10 +114,10 @@ print' opts@CliOpts{rawopts_=rawopts} j = do
   let
     -- lbl = lbl_ "print'"
     j' = j
-      -- & dbg9With (lbl "amounts before setting full precision".showJournalAmountsDebug)
+      -- & dbg9With (lbl "amounts before setting full precision".showJournalPostingAmountsDebug)
       & journalMapPostingAmounts mixedAmountSetFullPrecision
-      -- & dbg9With (lbl "amounts after  setting full precision: ".showJournalAmountsDebug)
-      & if boolopt "location" rawopts then journalMapTransactions addLocationTag else id
+      -- & dbg9With (lbl "amounts after  setting full precision: ".showJournalPostingAmountsDebug)
+      & if boolopt "locations" rawopts then journalMapTransactions addLocationTag else id
 
   case maybestringopt "match" $ rawopts_ opts of
     Nothing   -> printEntries opts j'
@@ -159,8 +160,7 @@ printEntries opts@CliOpts{rawopts_=rawopts, reportspec_=rspec} j =
           -- Use the fully inferred and amount-styled/rounded transaction in the following situations:
           -- with -x/--explicit:
           | boolopt "explicit" (rawopts_ opts) = id
-          -- with --show-costs:
-          -- XXX infer_costs is --infer-costs not --show-costs. And where is show-costs used anyway ?
+          -- with --infer-costs
           | opts ^. infer_costs = id
           -- with -B/-V/-X/--value ("because of #551, and because of print -V valuing only one posting when there's an implicit txn price.")
           | has (value . _Just) opts = id
